import struct

class Struct:
  def __init__(self,fields):
    fmt = '>'
    self._count = []
    self._fields = []
    for field in fields:
      fmt += str(field[0]) + field[1]
      if ( field[1] == 's' ): field[0] = 1
      if ( field[0] > 1 ):
        setattr(self,field[2],[])
        for i in range(field[0]): getattr(self,field[2]).append(field[3 + i])
      else: setattr(self,field[2],field[3])
      self._fields.append([field[0],field[2]])
    self._struct = struct.Struct(fmt)

  def write(self,f):
    data = []
    for field in self._fields:
      if ( field[0] > 1 ):
        for v in getattr(self,field[1]): data.append(v) 
      else: data.append(getattr(self,field[1]))
    f.write(self._struct.pack(*data))

class BinaryFile(file):
  """Base class for handling binary files."""

  def __init__(self,file_name,mode = 'r'): file.__init__(self,file_name,mode + 'b')

  def read_u8(self): return struct.unpack('>B',self.read(1))[0]
  def read_s16(self): return struct.unpack('>h',self.read(2))[0]
  def read_u16(self): return struct.unpack('>H',self.read(2))[0]
  def read_s32(self): return struct.unpack('>l',self.read(4))[0]
  def read_u32(self): return struct.unpack('>L',self.read(4))[0]
  def read_f32(self):
    x = self.read(4)
    x = '\0' * (4 - len(x)) + x
    return struct.unpack('>f',x)[0]

  def write_u8(self,v): self.write(struct.pack('>B',v))
  def write_s16(self,v): self.write(struct.pack('>h',v))
  def write_u16(self,v): self.write(struct.pack('>H',v))
  def write_s32(self,v): self.write(struct.pack('>l',v))
  def write_u32(self,v): self.write(struct.pack('>L',v))
  def write_f32(self,v): self.write(struct.pack('>f',v))

  def write_struct(s): s._write(self)

  def write_offset(self,base,at):
    pos = self.tell()
    self.seek(base + at)
    self.write_u32(pos - base)
    self.seek(pos)

  def write_cstring(self,s):
    self.write(s.decode('ascii'))
    self.write_u8(0)

  def align(self,length):
    while ( self.tell() % length != 0 ): self.write_u8(0xFF)
