import pyglet
from pyglet.gl import *
from pyglet.window import key
from ctypes import pointer, sizeof
import pickle

def gl_init(r, g, b):
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glClearDepth(1.0)
    glDepthFunc(GL_LESS)
    glEnable(GL_DEPTH_TEST)
    glShadeModel(GL_SMOOTH)
    # glMatrixMode(GL_PROJECTION)
    # glLoadIdentity()
    # gluPerspective(50.0 * 0.1, 800/600, 1, 20)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    glColor3f(r, g, b)
    glEnableClientState(GL_VERTEX_ARRAY)
    
    
def gl_create_vbo():
    vbo_id = GLuint()
    glGenBuffers(1, pointer(vbo_id))
    
    return vbo_id
    
def gl_set_vbo(vbo_id, data):
    data = (GLfloat*len(data))(*data)
    glBindBuffer(GL_ARRAY_BUFFER, vbo_id)
    glBufferData(GL_ARRAY_BUFFER, sizeof(data), 0, GL_DYNAMIC_DRAW)
    glBufferSubData(GL_ARRAY_BUFFER, 0, sizeof(data), data)
    
def gl_draw(num_vertices, dimensions, primitive):
    glVertexPointer(dimensions, GL_FLOAT, 0, 0)
    glDrawArrays(primitive, 0, num_vertices)
    
window = pyglet.window.Window(800, 600)
keys = dict(left=False, right=False)

cam_x = 100

def update(dt):
    global cam_x
    
    if keys['left'] == True:
        cam_x += 1
        glTranslatef(cam_x, 0, 0)
    elif keys['right'] == True:
        cam_x -= 1
        glTranslatef(cam_x, 0, 0)

@window.event
def on_draw():
    vertices = pickle.load(open('vertices.txt', 'rb'))
    gl_init(255, 255, 255)

    glTranslatef(cam_x, 0, 0)
    vbo_id = gl_create_vbo()
    for point in vertices:
        gl_set_vbo(vbo_id, point)
        gl_draw(1, 3, GL_POINTS)

    pyglet.clock.schedule_interval(update, 1/120.0)
def on_key_press(symbol, modifiers):
    if symbol == key.LEFT:
        keys['left'] = True
    elif symbol == key.RIGHT:
        keys['right'] = True
def on_key_release(symbol, modifiers):
    if symbol == key.LEFT:
        keys['left'] = False
    elif symbol == key.RIGHT:
        keys['right'] = False

window.push_handlers(on_draw)
window.push_handlers(on_key_press)
window.push_handlers(on_key_release)

if __name__ == '__main__':
    pyglet.app.run()
